unit ltr114_ProcessThreadUnit;

interface

uses
  Classes, SyncObjs,StdCtrls, SysUtils, Forms,
  ltrapi, ltr114api;

const
  LTR114THREAD_STATUS_WAIT        = 0;
  LTR114THREAD_STATUS_RUN       = 1;
  LTR114THREAD_STATUS_FORCE_STOP  = 2;
  LTR114THREAD_STATUS_STOPPED     = 3;
  LTR114THREAD_STATUS_ERROR       = 4;



type
  //  
  LTR114_ProcessThread = class(TThread)
  public
    LChEdits : array [0..LTR114_MAX_CHANNEL-1] of TEdit;
    ThermEdit : TEdit;
    ltr_correction_mode : integer;
    pltr114 : PTLTR114 ;

        //        2- 
    procedure SetLtrStatus(Status : integer);
    function GetLtrStatus() : integer;

    destructor Free();
    constructor Create(SuspendCreate : boolean);

  private
     { Private declarations }
    ddata : array [0..1999] of double;
    tdata : array [0..1999] of double;
    tcnt : integer;
    //     LTR114_STATUS   
    crit_sect: TCriticalSection;
    ltr114_status: integer; // 

    procedure UpdateForm;
  protected
    procedure Execute; override;
  end;

implementation
  //        
  constructor LTR114_ProcessThread.Create(SuspendCreate : boolean);
  begin
     Inherited Create(SuspendCreate);
     crit_sect := TCriticalSection.Create;
     SetLtrStatus(LTR114THREAD_STATUS_WAIT);
  end;

  //    
  procedure LTR114_ProcessThread.SetLtrStatus(Status : integer) ;
  begin
      crit_sect.Enter();
      ltr114_status := Status;
      crit_sect.Leave();
  end;

  //    
  function LTR114_ProcessThread.GetLtrStatus() : integer;
  var
      res : integer;
  begin
      crit_sect.Enter();
      res := ltr114_status;
      crit_sect.Leave();
      GetLtrStatus:= res;
  end;

 //    
 //(  syncronize     
procedure LTR114_ProcessThread.UpdateForm;
var
  i: integer;
begin
   for i:=0 to pltr114^.LChQnt-1 do
   begin
       LChEdits[i].Text := FloatToStr(ddata[i]);
   end;
   if ((pltr114^.SpecialFeatures and LTR114_FEATURES_THERM) <> 0) and (tcnt<>0) then
       ThermEdit.Text := FloatToStr(tdata[0]);
end;

procedure LTR114_ProcessThread.Execute;
var
  res,size: integer;
  data : array [0..3999] of cardinal;
  time : array [0..3999] of cardinal;

begin
  SetLtrStatus(LTR114THREAD_STATUS_RUN);
  res := LTR114_Calibrate(pltr114);
  if (res = LTR_OK) then
    res := LTR114_Start(pltr114);
  if (res = LTR_OK) then
  begin

     while (GetLtrStatus() = LTR114THREAD_STATUS_RUN) and (res = LTR_OK) do
     begin
        //   
        //   -        
        //  -      hltr114.FrameLength
        size := round(LTR114_CLOCK*1000/(LTR114_ADC_DIVIDER*pltr114^.FreqDivider*pltr114^.FrameLength));
        if (size<1) then
            size := 1;
        size := size * pltr114^.FrameLength;

        res := LTR114_Recv(pltr114, @data[0], @time[0], size, 2000);
        if (res > 0) then
        begin
          size := res;
          //   -    LTR114_ProcessDataTherm
          //: LTR114_ProcessDataTherm        
          if ((pltr114^.SpecialFeatures and LTR114_FEATURES_THERM) <> 0) then
              res := LTR114_ProcessDataTherm(pltr114, @data[0], @ddata[0], @tdata[0], @size, @tcnt, ltr_correction_mode, LTR114_PROCF_VALUE)     // 
          else
              res := LTR114_ProcessData(pltr114, @data[0], @ddata[0], @size, ltr_correction_mode, LTR114_PROCF_VALUE);
          self.Synchronize(UpdateForm);
        end;

     end;
     res := LTR114_Stop(pltr114);
  end;

  LTR114_Close(pltr114);


  if (res<>LTR_OK) then
  begin
    SetLtrStatus(LTR114THREAD_STATUS_ERROR);
    Application.MessageBox(LTR114_GetErrorString(res), 'LTR114 Error');
  end
  else
     SetLtrStatus(LTR114THREAD_STATUS_STOPPED);

end;

// .   
destructor LTR114_ProcessThread.Free();
begin
   crit_sect.Free();
   Inherited Free;
end;

end.
 